@@ -32,7 +32,7 @@ mod alignment;
 pub use alignment::ALIGNMENT;
 
 // If this number is not zero after all objects have been `drop`, there is a memory leak
-pub static mut ALLOCATIONS: AtomicIsize = AtomicIsize::new(0);
+static mut ALLOCATIONS: AtomicIsize = AtomicIsize::new(0);
 
 /// Returns the total number of bytes allocated to buffers by the allocator.
 pub fn total_allocated_bytes() -> isize {
