@@ -17,15 +17,16 @@ use std::any::Any;
 use std::fmt::Display;
 
 use crate::error::Result;
-use crate::types::days_ms;
+use crate::types::{days_ms, months_days_ns};
 use crate::{
     bitmap::{Bitmap, MutableBitmap},
-    datatypes::{DataType, IntervalUnit},
+    datatypes::DataType,
 };
 
 /// A trait representing an immutable Arrow array. Arrow arrays are trait objects
 /// that are infalibly downcasted to concrete types according to the [`Array::data_type`].
 pub trait Array: std::fmt::Debug + Send + Sync {
+    /// Convert to trait object.
     fn as_any(&self) -> &dyn Any;
 
     /// The length of the [`Array`]. Every array has a length corresponding to the number of
@@ -163,47 +164,67 @@ macro_rules! with_match_dictionary_key_type {(
     }
 })}
 
+macro_rules! with_match_physical_dictionary_key_type {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use crate::datatypes::DictionaryIndexType::*;
+    match $key_type {
+        Int8 => __with_ty__! { i8 },
+        Int16 => __with_ty__! { i16 },
+        Int32 => __with_ty__! { i32 },
+        Int64 => __with_ty__! { i64 },
+        UInt8 => __with_ty__! { u8 },
+        UInt16 => __with_ty__! { u16 },
+        UInt32 => __with_ty__! { u32 },
+        UInt64 => __with_ty__! { u64 },
+    }
+})}
+
+macro_rules! with_match_primitive_type {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use crate::datatypes::PrimitiveType::*;
+    use crate::types::{days_ms, months_days_ns};
+    match $key_type {
+        Int8 => __with_ty__! { i8 },
+        Int16 => __with_ty__! { i16 },
+        Int32 => __with_ty__! { i32 },
+        Int64 => __with_ty__! { i64 },
+        Int128 => __with_ty__! { i128 },
+        DaysMs => __with_ty__! { days_ms },
+        MonthDayNano => __with_ty__! { months_days_ns },
+        UInt8 => __with_ty__! { u8 },
+        UInt16 => __with_ty__! { u16 },
+        UInt32 => __with_ty__! { u32 },
+        UInt64 => __with_ty__! { u64 },
+        Float32 => __with_ty__! { f32 },
+        Float64 => __with_ty__! { f64 },
+    }
+})}
+
 impl Display for dyn Array {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self.data_type() {
-            DataType::Null => fmt_dyn!(self, NullArray, f),
-            DataType::Boolean => fmt_dyn!(self, BooleanArray, f),
-            DataType::Int8 => fmt_dyn!(self, PrimitiveArray<i8>, f),
-            DataType::Int16 => fmt_dyn!(self, PrimitiveArray<i16>, f),
-            DataType::Int32
-            | DataType::Date32
-            | DataType::Time32(_)
-            | DataType::Interval(IntervalUnit::YearMonth) => {
-                fmt_dyn!(self, PrimitiveArray<i32>, f)
-            }
-            DataType::Interval(IntervalUnit::DayTime) => {
-                fmt_dyn!(self, PrimitiveArray<days_ms>, f)
-            }
-            DataType::Int64
-            | DataType::Date64
-            | DataType::Time64(_)
-            | DataType::Timestamp(_, _)
-            | DataType::Duration(_) => fmt_dyn!(self, PrimitiveArray<i64>, f),
-            DataType::Decimal(_, _) => fmt_dyn!(self, PrimitiveArray<i128>, f),
-            DataType::UInt8 => fmt_dyn!(self, PrimitiveArray<u8>, f),
-            DataType::UInt16 => fmt_dyn!(self, PrimitiveArray<u16>, f),
-            DataType::UInt32 => fmt_dyn!(self, PrimitiveArray<u32>, f),
-            DataType::UInt64 => fmt_dyn!(self, PrimitiveArray<u64>, f),
-            DataType::Float16 => unreachable!(),
-            DataType::Float32 => fmt_dyn!(self, PrimitiveArray<f32>, f),
-            DataType::Float64 => fmt_dyn!(self, PrimitiveArray<f64>, f),
-            DataType::Binary => fmt_dyn!(self, BinaryArray<i32>, f),
-            DataType::LargeBinary => fmt_dyn!(self, BinaryArray<i64>, f),
-            DataType::FixedSizeBinary(_) => fmt_dyn!(self, FixedSizeBinaryArray, f),
-            DataType::Utf8 => fmt_dyn!(self, Utf8Array::<i32>, f),
-            DataType::LargeUtf8 => fmt_dyn!(self, Utf8Array::<i64>, f),
-            DataType::List(_) => fmt_dyn!(self, ListArray::<i32>, f),
-            DataType::LargeList(_) => fmt_dyn!(self, ListArray::<i64>, f),
-            DataType::FixedSizeList(_, _) => fmt_dyn!(self, FixedSizeListArray, f),
-            DataType::Struct(_) => fmt_dyn!(self, StructArray, f),
-            DataType::Union(_, _, _) => fmt_dyn!(self, UnionArray, f),
-            DataType::Dictionary(key_type, _) => {
-                with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+        use crate::datatypes::PhysicalType::*;
+        match self.data_type().to_physical_type() {
+            Null => fmt_dyn!(self, NullArray, f),
+            Boolean => fmt_dyn!(self, BooleanArray, f),
+            Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
+                fmt_dyn!(self, PrimitiveArray<$T>, f)
+            }),
+            Binary => fmt_dyn!(self, BinaryArray<i32>, f),
+            LargeBinary => fmt_dyn!(self, BinaryArray<i64>, f),
+            FixedSizeBinary => fmt_dyn!(self, FixedSizeBinaryArray, f),
+            Utf8 => fmt_dyn!(self, Utf8Array::<i32>, f),
+            LargeUtf8 => fmt_dyn!(self, Utf8Array::<i64>, f),
+            List => fmt_dyn!(self, ListArray::<i32>, f),
+            LargeList => fmt_dyn!(self, ListArray::<i64>, f),
+            FixedSizeList => fmt_dyn!(self, FixedSizeListArray, f),
+            Struct => fmt_dyn!(self, StructArray, f),
+            Union => fmt_dyn!(self, UnionArray, f),
+            Dictionary(key_type) => {
+                with_match_physical_dictionary_key_type!(key_type, |$T| {
                     fmt_dyn!(self, DictionaryArray::<$T>, f)
                 })
             }
@@ -213,46 +234,26 @@ impl Display for dyn Array {
 
 /// Creates a new [`Array`] with a [`Array::len`] of 0.
 pub fn new_empty_array(data_type: DataType) -> Box<dyn Array> {
-    match data_type {
-        DataType::Null => Box::new(NullArray::new_empty()),
-        DataType::Boolean => Box::new(BooleanArray::new_empty()),
-        DataType::Int8 => Box::new(PrimitiveArray::<i8>::new_empty(data_type)),
-        DataType::Int16 => Box::new(PrimitiveArray::<i16>::new_empty(data_type)),
-        DataType::Int32
-        | DataType::Date32
-        | DataType::Time32(_)
-        | DataType::Interval(IntervalUnit::YearMonth) => {
-            Box::new(PrimitiveArray::<i32>::new_empty(data_type))
-        }
-        DataType::Interval(IntervalUnit::DayTime) => {
-            Box::new(PrimitiveArray::<days_ms>::new_empty(data_type))
-        }
-        DataType::Int64
-        | DataType::Date64
-        | DataType::Time64(_)
-        | DataType::Timestamp(_, _)
-        | DataType::Duration(_) => Box::new(PrimitiveArray::<i64>::new_empty(data_type)),
-        DataType::Decimal(_, _) => Box::new(PrimitiveArray::<i128>::new_empty(data_type)),
-        DataType::UInt8 => Box::new(PrimitiveArray::<u8>::new_empty(data_type)),
-        DataType::UInt16 => Box::new(PrimitiveArray::<u16>::new_empty(data_type)),
-        DataType::UInt32 => Box::new(PrimitiveArray::<u32>::new_empty(data_type)),
-        DataType::UInt64 => Box::new(PrimitiveArray::<u64>::new_empty(data_type)),
-        DataType::Float16 => unreachable!(),
-        DataType::Float32 => Box::new(PrimitiveArray::<f32>::new_empty(data_type)),
-        DataType::Float64 => Box::new(PrimitiveArray::<f64>::new_empty(data_type)),
-        DataType::Binary => Box::new(BinaryArray::<i32>::new_empty()),
-        DataType::LargeBinary => Box::new(BinaryArray::<i64>::new_empty()),
-        DataType::FixedSizeBinary(_) => Box::new(FixedSizeBinaryArray::new_empty(data_type)),
-        DataType::Utf8 => Box::new(Utf8Array::<i32>::new_empty()),
-        DataType::LargeUtf8 => Box::new(Utf8Array::<i64>::new_empty()),
-        DataType::List(_) => Box::new(ListArray::<i32>::new_empty(data_type)),
-        DataType::LargeList(_) => Box::new(ListArray::<i64>::new_empty(data_type)),
-        DataType::FixedSizeList(_, _) => Box::new(FixedSizeListArray::new_empty(data_type)),
-        DataType::Struct(fields) => Box::new(StructArray::new_empty(&fields)),
-        DataType::Union(_, _, _) => Box::new(UnionArray::new_empty(data_type)),
-        DataType::Dictionary(key_type, value_type) => {
-            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
-                Box::new(DictionaryArray::<$T>::new_empty(*value_type))
+    use crate::datatypes::PhysicalType::*;
+    match data_type.to_physical_type() {
+        Null => Box::new(NullArray::new_empty(data_type)),
+        Boolean => Box::new(BooleanArray::new_empty(data_type)),
+        Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
+            Box::new(PrimitiveArray::<$T>::new_empty(data_type))
+        }),
+        Binary => Box::new(BinaryArray::<i32>::new_empty(data_type)),
+        LargeBinary => Box::new(BinaryArray::<i64>::new_empty(data_type)),
+        FixedSizeBinary => Box::new(FixedSizeBinaryArray::new_empty(data_type)),
+        Utf8 => Box::new(Utf8Array::<i32>::new_empty(data_type)),
+        LargeUtf8 => Box::new(Utf8Array::<i64>::new_empty(data_type)),
+        List => Box::new(ListArray::<i32>::new_empty(data_type)),
+        LargeList => Box::new(ListArray::<i64>::new_empty(data_type)),
+        FixedSizeList => Box::new(FixedSizeListArray::new_empty(data_type)),
+        Struct => Box::new(StructArray::new_empty(data_type)),
+        Union => Box::new(UnionArray::new_empty(data_type)),
+        Dictionary(key_type) => {
+            with_match_physical_dictionary_key_type!(key_type, |$T| {
+                Box::new(DictionaryArray::<$T>::new_empty(data_type))
             })
         }
     }
@@ -262,46 +263,26 @@ pub fn new_empty_array(data_type: DataType) -> Box<dyn Array> {
 /// The array is guaranteed to have [`Array::null_count`] equal to [`Array::len`]
 /// for all types except Union, which does not have a validity.
 pub fn new_null_array(data_type: DataType, length: usize) -> Box<dyn Array> {
-    match data_type {
-        DataType::Null => Box::new(NullArray::new_null(length)),
-        DataType::Boolean => Box::new(BooleanArray::new_null(length)),
-        DataType::Int8 => Box::new(PrimitiveArray::<i8>::new_null(data_type, length)),
-        DataType::Int16 => Box::new(PrimitiveArray::<i16>::new_null(data_type, length)),
-        DataType::Int32
-        | DataType::Date32
-        | DataType::Time32(_)
-        | DataType::Interval(IntervalUnit::YearMonth) => {
-            Box::new(PrimitiveArray::<i32>::new_null(data_type, length))
-        }
-        DataType::Interval(IntervalUnit::DayTime) => {
-            Box::new(PrimitiveArray::<days_ms>::new_null(data_type, length))
-        }
-        DataType::Int64
-        | DataType::Date64
-        | DataType::Time64(_)
-        | DataType::Timestamp(_, _)
-        | DataType::Duration(_) => Box::new(PrimitiveArray::<i64>::new_null(data_type, length)),
-        DataType::Decimal(_, _) => Box::new(PrimitiveArray::<i128>::new_null(data_type, length)),
-        DataType::UInt8 => Box::new(PrimitiveArray::<u8>::new_null(data_type, length)),
-        DataType::UInt16 => Box::new(PrimitiveArray::<u16>::new_null(data_type, length)),
-        DataType::UInt32 => Box::new(PrimitiveArray::<u32>::new_null(data_type, length)),
-        DataType::UInt64 => Box::new(PrimitiveArray::<u64>::new_null(data_type, length)),
-        DataType::Float16 => unreachable!(),
-        DataType::Float32 => Box::new(PrimitiveArray::<f32>::new_null(data_type, length)),
-        DataType::Float64 => Box::new(PrimitiveArray::<f64>::new_null(data_type, length)),
-        DataType::Binary => Box::new(BinaryArray::<i32>::new_null(length)),
-        DataType::LargeBinary => Box::new(BinaryArray::<i64>::new_null(length)),
-        DataType::FixedSizeBinary(_) => Box::new(FixedSizeBinaryArray::new_null(data_type, length)),
-        DataType::Utf8 => Box::new(Utf8Array::<i32>::new_null(length)),
-        DataType::LargeUtf8 => Box::new(Utf8Array::<i64>::new_null(length)),
-        DataType::List(_) => Box::new(ListArray::<i32>::new_null(data_type, length)),
-        DataType::LargeList(_) => Box::new(ListArray::<i64>::new_null(data_type, length)),
-        DataType::FixedSizeList(_, _) => Box::new(FixedSizeListArray::new_null(data_type, length)),
-        DataType::Struct(fields) => Box::new(StructArray::new_null(&fields, length)),
-        DataType::Union(_, _, _) => Box::new(UnionArray::new_null(data_type, length)),
-        DataType::Dictionary(key_type, value_type) => {
-            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
-                Box::new(DictionaryArray::<$T>::new_null(*value_type, length))
+    use crate::datatypes::PhysicalType::*;
+    match data_type.to_physical_type() {
+        Null => Box::new(NullArray::new_null(data_type, length)),
+        Boolean => Box::new(BooleanArray::new_null(data_type, length)),
+        Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
+            Box::new(PrimitiveArray::<$T>::new_null(data_type, length))
+        }),
+        Binary => Box::new(BinaryArray::<i32>::new_null(data_type, length)),
+        LargeBinary => Box::new(BinaryArray::<i64>::new_null(data_type, length)),
+        FixedSizeBinary => Box::new(FixedSizeBinaryArray::new_null(data_type, length)),
+        Utf8 => Box::new(Utf8Array::<i32>::new_null(data_type, length)),
+        LargeUtf8 => Box::new(Utf8Array::<i64>::new_null(data_type, length)),
+        List => Box::new(ListArray::<i32>::new_null(data_type, length)),
+        LargeList => Box::new(ListArray::<i64>::new_null(data_type, length)),
+        FixedSizeList => Box::new(FixedSizeListArray::new_null(data_type, length)),
+        Struct => Box::new(StructArray::new_null(data_type, length)),
+        Union => Box::new(UnionArray::new_null(data_type, length)),
+        Dictionary(key_type) => {
+            with_match_physical_dictionary_key_type!(key_type, |$T| {
+                Box::new(DictionaryArray::<$T>::new_null(data_type, length))
             })
         }
     }
@@ -319,43 +300,25 @@ macro_rules! clone_dyn {
 /// This operation is `O(1)` over `len`, as it amounts to increase two ref counts
 /// and moving the concrete struct under a `Box`.
 pub fn clone(array: &dyn Array) -> Box<dyn Array> {
-    match array.data_type() {
-        DataType::Null => clone_dyn!(array, NullArray),
-        DataType::Boolean => clone_dyn!(array, BooleanArray),
-        DataType::Int8 => clone_dyn!(array, PrimitiveArray<i8>),
-        DataType::Int16 => clone_dyn!(array, PrimitiveArray<i16>),
-        DataType::Int32
-        | DataType::Date32
-        | DataType::Time32(_)
-        | DataType::Interval(IntervalUnit::YearMonth) => {
-            clone_dyn!(array, PrimitiveArray<i32>)
-        }
-        DataType::Interval(IntervalUnit::DayTime) => clone_dyn!(array, PrimitiveArray<days_ms>),
-        DataType::Int64
-        | DataType::Date64
-        | DataType::Time64(_)
-        | DataType::Timestamp(_, _)
-        | DataType::Duration(_) => clone_dyn!(array, PrimitiveArray<i64>),
-        DataType::Decimal(_, _) => clone_dyn!(array, PrimitiveArray<i128>),
-        DataType::UInt8 => clone_dyn!(array, PrimitiveArray<u8>),
-        DataType::UInt16 => clone_dyn!(array, PrimitiveArray<u16>),
-        DataType::UInt32 => clone_dyn!(array, PrimitiveArray<u32>),
-        DataType::UInt64 => clone_dyn!(array, PrimitiveArray<u64>),
-        DataType::Float16 => unreachable!(),
-        DataType::Float32 => clone_dyn!(array, PrimitiveArray<f32>),
-        DataType::Float64 => clone_dyn!(array, PrimitiveArray<f64>),
-        DataType::Binary => clone_dyn!(array, BinaryArray<i32>),
-        DataType::LargeBinary => clone_dyn!(array, BinaryArray<i64>),
-        DataType::FixedSizeBinary(_) => clone_dyn!(array, FixedSizeBinaryArray),
-        DataType::Utf8 => clone_dyn!(array, Utf8Array::<i32>),
-        DataType::LargeUtf8 => clone_dyn!(array, Utf8Array::<i64>),
-        DataType::List(_) => clone_dyn!(array, ListArray::<i32>),
-        DataType::LargeList(_) => clone_dyn!(array, ListArray::<i64>),
-        DataType::FixedSizeList(_, _) => clone_dyn!(array, FixedSizeListArray),
-        DataType::Struct(_) => clone_dyn!(array, StructArray),
-        DataType::Union(_, _, _) => clone_dyn!(array, UnionArray),
-        DataType::Dictionary(key_type, _) => {
-            with_match_dictionary_key_type!(key_type.as_ref(), |$T| {
+    use crate::datatypes::PhysicalType::*;
+    match array.data_type().to_physical_type() {
+        Null => clone_dyn!(array, NullArray),
+        Boolean => clone_dyn!(array, BooleanArray),
+        Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
+            clone_dyn!(array, PrimitiveArray<$T>)
+        }),
+        Binary => clone_dyn!(array, BinaryArray<i32>),
+        LargeBinary => clone_dyn!(array, BinaryArray<i64>),
+        FixedSizeBinary => clone_dyn!(array, FixedSizeBinaryArray),
+        Utf8 => clone_dyn!(array, Utf8Array::<i32>),
+        LargeUtf8 => clone_dyn!(array, Utf8Array::<i64>),
+        List => clone_dyn!(array, ListArray::<i32>),
+        LargeList => clone_dyn!(array, ListArray::<i64>),
+        FixedSizeList => clone_dyn!(array, FixedSizeListArray),
+        Struct => clone_dyn!(array, StructArray),
+        Union => clone_dyn!(array, UnionArray),
+        Dictionary(key_type) => {
+            with_match_physical_dictionary_key_type!(key_type, |$T| {
                 clone_dyn!(array, DictionaryArray::<$T>)
             })
         }
@@ -404,6 +367,7 @@ pub use self::ffi::ToFfi;
 /// A trait describing the ability of a struct to create itself from a iterator.
 /// This is similar to [`Extend`], but accepted the creation to error.
 pub trait TryExtend<A> {
+    /// Fallible version of [`Extend::extend`].
     fn try_extend<I: IntoIterator<Item = A>>(&mut self, iter: I) -> Result<()>;
 }
 
@@ -468,10 +432,14 @@ pub trait IterableListArray: Array {
 /// 2. `offsets[i] >= offsets[i-1] for all i`
 /// 3. `offsets[i] < values.len() for all i`
 pub unsafe trait GenericBinaryArray<O: Offset>: Array {
+    /// The values of the array
     fn values(&self) -> &[u8];
+    /// The offsets of the array
     fn offsets(&self) -> &[O];
 }
 
 // backward compatibility
 use std::sync::Arc;
+
+/// A type def of [`Array`].
 pub type ArrayRef = Arc<dyn Array>;
