@@ -17,6 +17,7 @@ use super::Utf8Array;
 /// The mutable version of [`Utf8Array`]. See [`MutableArray`] for more details.
 #[derive(Debug)]
 pub struct MutableUtf8Array<O: Offset> {
+    data_type: DataType,
     offsets: MutableBuffer<O>,
     values: MutableBuffer<u8>,
     validity: Option<MutableBitmap>,
@@ -25,6 +26,7 @@ pub struct MutableUtf8Array<O: Offset> {
 impl<O: Offset> From<MutableUtf8Array<O>> for Utf8Array<O> {
     fn from(other: MutableUtf8Array<O>) -> Self {
         Utf8Array::<O>::from_data(
+            other.data_type,
             other.offsets.into(),
             other.values.into(),
             other.validity.map(|x| x.into()),
@@ -44,6 +46,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         let mut offsets = MutableBuffer::<O>::new();
         offsets.push(O::default());
         Self {
+            data_type: Self::default_data_type(),
             offsets,
             values: MutableBuffer::<u8>::new(),
             validity: None,
@@ -57,6 +60,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// * The `values` between `offsets` are not utf8 encoded
     /// * The validity is not `None` and its length is different from `offsets`'s length minus one.
     pub fn from_data(
+        data_type: DataType,
         offsets: MutableBuffer<O>,
         values: MutableBuffer<u8>,
         validity: Option<MutableBitmap>,
@@ -65,7 +69,11 @@ impl<O: Offset> MutableUtf8Array<O> {
         if let Some(ref validity) = validity {
             assert_eq!(offsets.len() - 1, validity.len());
         }
+        if data_type.to_physical_type() != Self::default_data_type().to_physical_type() {
+            panic!("MutableUtf8Array can only be initialized with DataType::Utf8 or DataType::LargeUtf8")
+        }
         Self {
+            data_type,
             offsets,
             values,
             validity,
@@ -80,6 +88,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// * The `offsets` and `values` are inconsistent
     /// * The validity is not `None` and its length is different from `offsets`'s length minus one.
     pub unsafe fn from_data_unchecked(
+        data_type: DataType,
         offsets: MutableBuffer<O>,
         values: MutableBuffer<u8>,
         validity: Option<MutableBitmap>,
@@ -88,13 +97,21 @@ impl<O: Offset> MutableUtf8Array<O> {
         if let Some(ref validity) = validity {
             assert_eq!(offsets.len() - 1, validity.len());
         }
+        if data_type.to_physical_type() != Self::default_data_type().to_physical_type() {
+            panic!("MutableUtf8Array can only be initialized with DataType::Utf8 or DataType::LargeUtf8")
+        }
         Self {
+            data_type,
             offsets,
             values,
             validity,
         }
     }
 
+    fn default_data_type() -> DataType {
+        Utf8Array::<O>::default_data_type()
+    }
+
     /// Initializes a new [`MutableUtf8Array`] with a pre-allocated capacity of slots.
     pub fn with_capacity(capacity: usize) -> Self {
         Self::with_capacities(capacity, 0)
@@ -106,6 +123,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         offsets.push(O::default());
 
         Self {
+            data_type: Self::default_data_type(),
             offsets,
             values: MutableBuffer::<u8>::with_capacity(values),
             validity: None,
@@ -171,6 +189,7 @@ impl<O: Offset> MutableArray for MutableUtf8Array<O> {
 
     fn as_arc(&mut self) -> Arc<dyn Array> {
         Arc::new(Utf8Array::from_data(
+            Self::default_data_type(),
             std::mem::take(&mut self.offsets).into(),
             std::mem::take(&mut self.values).into(),
             std::mem::take(&mut self.validity).map(|x| x.into()),
@@ -218,7 +237,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         let (validity, offsets, values) = trusted_len_unzip(iterator);
 
         // soundness: P is `str`
-        Self::from_data_unchecked(offsets, values, validity)
+        Self::from_data_unchecked(Self::default_data_type(), offsets, values, validity)
     }
 
     /// Creates a [`MutableUtf8Array`] from an iterator of trusted length.
@@ -240,7 +259,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         // soundness: I is `TrustedLen`
         let (offsets, values) = unsafe { trusted_len_values_iter(iterator) };
         // soundness: T is AsRef<str>
-        unsafe { Self::from_data_unchecked(offsets, values, None) }
+        unsafe { Self::from_data_unchecked(Self::default_data_type(), offsets, values, None) }
     }
 
     /// Creates a new [`MutableUtf8Array`] from an iterator.
@@ -275,7 +294,12 @@ impl<O: Offset> MutableUtf8Array<O> {
         let (validity, offsets, values) = try_trusted_len_unzip(iterator)?;
 
         // soundness: P is `str`
-        Ok(Self::from_data_unchecked(offsets, values, validity))
+        Ok(Self::from_data_unchecked(
+            Self::default_data_type(),
+            offsets,
+            values,
+            validity,
+        ))
     }
 
     /// Creates a [`MutableUtf8Array`] from an falible iterator of trusted length.
@@ -293,7 +317,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     pub fn from_iter_values<T: AsRef<str>, I: Iterator<Item = T>>(iterator: I) -> Self {
         let (offsets, values) = values_iter(iterator);
         // soundness: T: AsRef<str>
-        unsafe { Self::from_data_unchecked(offsets, values, None) }
+        unsafe { Self::from_data_unchecked(Self::default_data_type(), offsets, values, None) }
     }
 }
 
