@@ -1,3 +1,4 @@
+use crate::datatypes::DataType;
 use crate::error::{ArrowError, Result};
 use crate::{
     array::{Array, BooleanArray},
@@ -87,7 +88,11 @@ pub fn or(lhs: &BooleanArray, rhs: &BooleanArray) -> Result<BooleanArray> {
         }
         (None, None) => None,
     };
-    Ok(BooleanArray::from_data(lhs_values | rhs_values, validity))
+    Ok(BooleanArray::from_data(
+        DataType::Boolean,
+        lhs_values | rhs_values,
+        validity,
+    ))
 }
 
 /// Logical 'and' with [Kleene logic](https://en.wikipedia.org/wiki/Three-valued_logic#Kleene_and_Priest_logics)
@@ -172,139 +177,9 @@ pub fn and(lhs: &BooleanArray, rhs: &BooleanArray) -> Result<BooleanArray> {
         }
         (None, None) => None,
     };
-    Ok(BooleanArray::from_data(lhs_values & rhs_values, validity))
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn and_generic() {
-        let lhs = BooleanArray::from(&[
-            None,
-            None,
-            None,
-            Some(false),
-            Some(false),
-            Some(false),
-            Some(true),
-            Some(true),
-            Some(true),
-        ]);
-        let rhs = BooleanArray::from(&[
-            None,
-            Some(false),
-            Some(true),
-            None,
-            Some(false),
-            Some(true),
-            None,
-            Some(false),
-            Some(true),
-        ]);
-        let c = and(&lhs, &rhs).unwrap();
-
-        let expected = BooleanArray::from(&[
-            None,
-            Some(false),
-            None,
-            Some(false),
-            Some(false),
-            Some(false),
-            None,
-            Some(false),
-            Some(true),
-        ]);
-
-        assert_eq!(c, expected);
-    }
-
-    #[test]
-    fn or_generic() {
-        let a = BooleanArray::from(&[
-            None,
-            None,
-            None,
-            Some(false),
-            Some(false),
-            Some(false),
-            Some(true),
-            Some(true),
-            Some(true),
-        ]);
-        let b = BooleanArray::from(&[
-            None,
-            Some(false),
-            Some(true),
-            None,
-            Some(false),
-            Some(true),
-            None,
-            Some(false),
-            Some(true),
-        ]);
-        let c = or(&a, &b).unwrap();
-
-        let expected = BooleanArray::from(&[
-            None,
-            None,
-            Some(true),
-            None,
-            Some(false),
-            Some(true),
-            Some(true),
-            Some(true),
-            Some(true),
-        ]);
-
-        assert_eq!(c, expected);
-    }
-
-    #[test]
-    fn or_right_nulls() {
-        let a = BooleanArray::from_slice(&[false, false, false, true, true, true]);
-
-        let b = BooleanArray::from(&[Some(true), Some(false), None, Some(true), Some(false), None]);
-
-        let c = or(&a, &b).unwrap();
-
-        let expected = BooleanArray::from(&[
-            Some(true),
-            Some(false),
-            None,
-            Some(true),
-            Some(true),
-            Some(true),
-        ]);
-
-        assert_eq!(c, expected);
-    }
-
-    #[test]
-    fn or_left_nulls() {
-        let a = BooleanArray::from(vec![
-            Some(true),
-            Some(false),
-            None,
-            Some(true),
-            Some(false),
-            None,
-        ]);
-
-        let b = BooleanArray::from_slice(&[false, false, false, true, true, true]);
-
-        let c = or(&a, &b).unwrap();
-
-        let expected = BooleanArray::from(vec![
-            Some(true),
-            Some(false),
-            None,
-            Some(true),
-            Some(true),
-            Some(true),
-        ]);
-
-        assert_eq!(c, expected);
-    }
+    Ok(BooleanArray::from_data(
+        DataType::Boolean,
+        lhs_values & rhs_values,
+        validity,
+    ))
 }
