@@ -1,17 +1,14 @@
-//! This module contains logical types defined in the
-//! [Arrow specification](https://arrow.apache.org/docs/cpp/api/datatype.html):
-//!
-//! * [`DataType`]
-//! * [`Field`]
-//! * [`Schema`]
-//! * [`TimeUnit`]
-//! * [`IntervalUnit`]
+//! Metadata declarations such as [`DataType`], [`Field`] and [`Schema`].
 mod field;
+mod physical_type;
 mod schema;
 
 pub use field::Field;
+pub use physical_type::*;
 pub use schema::Schema;
 
+pub(crate) use field::{get_extension, Extension, Metadata};
+
 /// The set of datatypes that are supported by this implementation of Apache Arrow.
 ///
 /// The Arrow specification on data types includes some more types.
@@ -27,7 +24,7 @@ pub use schema::Schema;
 /// Nested types can themselves be nested within other arrays.
 /// For more information on these types please see
 /// [the physical memory layout of Apache Arrow](https://arrow.apache.org/docs/format/Columnar.html#physical-memory-layout).
-#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum DataType {
     /// Null type, representing an array without values or validity, only a length.
     Null,
@@ -121,6 +118,8 @@ pub enum DataType {
     /// scale is the number of decimal places.
     /// The number 999.99 has a precision of 5 and scale of 2.
     Decimal(usize, usize),
+    /// Extension type.
+    Extension(String, Box<DataType>, Option<String>),
 }
 
 impl std::fmt::Display for DataType {
@@ -129,8 +128,8 @@ impl std::fmt::Display for DataType {
     }
 }
 
-/// An absolute length of time in seconds, milliseconds, microseconds or nanoseconds.
-#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+/// Time units defined in Arrow.
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum TimeUnit {
     /// Time in seconds.
     Second,
@@ -142,14 +141,21 @@ pub enum TimeUnit {
     Nanosecond,
 }
 
-/// YEAR_MONTH or DAY_TIME interval in SQL style.
-#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+/// Interval units defined in Arrow
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum IntervalUnit {
     /// Indicates the number of elapsed whole months, stored as 4-byte integers.
     YearMonth,
     /// Indicates the number of elapsed days and milliseconds,
     /// stored as 2 contiguous 32-bit integers (8-bytes in total).
     DayTime,
+    /// The values are stored contiguously in 16 byte blocks. Months and
+    /// days are encoded as 32 bit integers and nanoseconds is encoded as a
+    /// 64 bit integer. All integers are signed. Each field is independent
+    /// (e.g. there is no constraint that nanoseconds have the same sign
+    /// as days or that the quantitiy of nanoseconds represents less
+    /// then a day's worth of time).
+    MonthDayNano,
 }
 
 impl DataType {
@@ -176,8 +182,64 @@ impl DataType {
             _ => self == other,
         }
     }
+
+    /// the [`PhysicalType`] of this [`DataType`].
+    pub fn to_physical_type(&self) -> PhysicalType {
+        use DataType::*;
+        match self {
+            Null => PhysicalType::Null,
+            Boolean => PhysicalType::Boolean,
+            Int8 => PhysicalType::Primitive(PrimitiveType::Int8),
+            Int16 => PhysicalType::Primitive(PrimitiveType::Int16),
+            Int32 | Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => {
+                PhysicalType::Primitive(PrimitiveType::Int32)
+            }
+            Int64 | Date64 | Timestamp(_, _) | Time64(_) | Duration(_) => {
+                PhysicalType::Primitive(PrimitiveType::Int64)
+            }
+            Decimal(_, _) => PhysicalType::Primitive(PrimitiveType::Int128),
+            UInt8 => PhysicalType::Primitive(PrimitiveType::UInt8),
+            UInt16 => PhysicalType::Primitive(PrimitiveType::UInt16),
+            UInt32 => PhysicalType::Primitive(PrimitiveType::UInt32),
+            UInt64 => PhysicalType::Primitive(PrimitiveType::UInt64),
+            Float16 => unreachable!(),
+            Float32 => PhysicalType::Primitive(PrimitiveType::Float32),
+            Float64 => PhysicalType::Primitive(PrimitiveType::Float64),
+            Interval(IntervalUnit::DayTime) => PhysicalType::Primitive(PrimitiveType::DaysMs),
+            Interval(IntervalUnit::MonthDayNano) => {
+                PhysicalType::Primitive(PrimitiveType::MonthDayNano)
+            }
+            Binary => PhysicalType::Binary,
+            FixedSizeBinary(_) => PhysicalType::FixedSizeBinary,
+            LargeBinary => PhysicalType::LargeBinary,
+            Utf8 => PhysicalType::Utf8,
+            LargeUtf8 => PhysicalType::LargeUtf8,
+            List(_) => PhysicalType::List,
+            FixedSizeList(_, _) => PhysicalType::FixedSizeList,
+            LargeList(_) => PhysicalType::LargeList,
+            Struct(_) => PhysicalType::Struct,
+            Union(_, _, _) => PhysicalType::Union,
+            Dictionary(key, _) => PhysicalType::Dictionary(to_dictionary_index_type(key.as_ref())),
+            Extension(_, key, _) => key.to_physical_type(),
+        }
+    }
+}
+
+fn to_dictionary_index_type(data_type: &DataType) -> DictionaryIndexType {
+    match data_type {
+        DataType::Int8 => DictionaryIndexType::Int8,
+        DataType::Int16 => DictionaryIndexType::Int16,
+        DataType::Int32 => DictionaryIndexType::Int32,
+        DataType::Int64 => DictionaryIndexType::Int64,
+        DataType::UInt8 => DictionaryIndexType::UInt8,
+        DataType::UInt16 => DictionaryIndexType::UInt16,
+        DataType::UInt32 => DictionaryIndexType::UInt32,
+        DataType::UInt64 => DictionaryIndexType::UInt64,
+        _ => ::core::unreachable!("A dictionary key type can only be of integer types"),
+    }
 }
 
 // backward compatibility
 use std::sync::Arc;
+/// typedef for [`Arc<Schema>`].
 pub type SchemaRef = Arc<Schema>;
