@@ -1,6 +1,6 @@
 use futures::{pin_mut, Stream, StreamExt};
 use parquet2::{
-    encoding::{bitpacking, hybrid_rle, uleb128, Encoding},
+    encoding::{hybrid_rle, Encoding},
     page::{DataPage, FixedLenByteArrayPageDict},
     read::StreamingIterator,
 };
@@ -21,14 +21,14 @@ use super::utils;
 pub(crate) fn read_dict_buffer(
     validity_buffer: &[u8],
     indices_buffer: &[u8],
-    length: u32,
+    additional: usize,
     size: i32,
     dict: &FixedLenByteArrayPageDict,
     values: &mut MutableBuffer<u8>,
     validity: &mut MutableBitmap,
 ) {
-    let length = length as usize;
     let size = size as usize;
+    let length = values.len() * size + additional;
     let dict_values = dict.values();
 
     // SPEC: Data page format: the bit width used to encode the entry ids stored as 1 byte (max bit width = 32),
@@ -36,20 +36,14 @@ pub(crate) fn read_dict_buffer(
     let bit_width = indices_buffer[0];
     let indices_buffer = &indices_buffer[1..];
 
-    let (_, consumed) = uleb128::decode(indices_buffer);
-    let indices_buffer = &indices_buffer[consumed..];
-
-    let non_null_indices_len = (indices_buffer.len() * 8 / bit_width as usize) as u32;
-
-    let mut indices =
-        bitpacking::Decoder::new(indices_buffer, bit_width, non_null_indices_len as usize);
+    let mut indices = hybrid_rle::HybridRleDecoder::new(indices_buffer, bit_width as u32, length);
 
     let validity_iterator = hybrid_rle::Decoder::new(validity_buffer, 1);
 
     for run in validity_iterator {
         match run {
             hybrid_rle::HybridEncoded::Bitpacked(packed) => {
-                let remaining = length - values.len();
+                let remaining = length - values.len() * size;
                 let len = std::cmp::min(packed.len() * 8, remaining);
                 for is_valid in BitmapIter::new(packed, 0, len) {
                     validity.push(is_valid);
@@ -80,13 +74,13 @@ pub(crate) fn read_dict_buffer(
 pub(crate) fn read_optional(
     validity_buffer: &[u8],
     values_buffer: &[u8],
-    length: u32,
+    additional: usize,
     size: i32,
     values: &mut MutableBuffer<u8>,
     validity: &mut MutableBitmap,
 ) {
-    let length = length as usize;
     let size = size as usize;
+    let length = values.len() * size + additional;
 
     assert_eq!(values_buffer.len() % size, 0);
     let mut values_iterator = values_buffer.chunks_exact(size);
@@ -97,7 +91,7 @@ pub(crate) fn read_optional(
         match run {
             hybrid_rle::HybridEncoded::Bitpacked(packed) => {
                 // the pack may contain more items than needed.
-                let remaining = length - values.len();
+                let remaining = length - values.len() * size;
                 let len = std::cmp::min(packed.len() * 8, remaining);
                 for is_valid in BitmapIter::new(packed, 0, len) {
                     validity.push(is_valid);
@@ -125,8 +119,13 @@ pub(crate) fn read_optional(
     }
 }
 
-pub(crate) fn read_required(buffer: &[u8], length: u32, size: i32, values: &mut MutableBuffer<u8>) {
-    assert_eq!(buffer.len(), length as usize * size as usize);
+pub(crate) fn read_required(
+    buffer: &[u8],
+    additional: usize,
+    size: i32,
+    values: &mut MutableBuffer<u8>,
+) {
+    assert_eq!(buffer.len(), additional * size as usize);
     values.extend_from_slice(buffer);
 }
 
@@ -200,33 +199,34 @@ pub(crate) fn extend_from_page(
     values: &mut MutableBuffer<u8>,
     validity: &mut MutableBitmap,
 ) -> Result<()> {
+    let additional = page.num_values();
     assert_eq!(descriptor.max_rep_level(), 0);
     assert!(descriptor.max_def_level() <= 1);
     let is_optional = descriptor.max_def_level() == 1;
 
-    let (validity_buffer, values_buffer, version) = utils::split_buffer(page, is_optional);
+    let (_, validity_buffer, values_buffer, version) = utils::split_buffer(page, descriptor);
 
     match (page.encoding(), page.dictionary_page(), is_optional) {
         (Encoding::PlainDictionary, Some(dict), true) => read_dict_buffer(
             validity_buffer,
             values_buffer,
-            page.num_values() as u32,
+            additional,
             size,
             dict.as_any().downcast_ref().unwrap(),
             values,
             validity,
         ),
-        (Encoding::Plain, None, true) => read_optional(
+        (Encoding::Plain, _, true) => read_optional(
             validity_buffer,
             values_buffer,
-            page.num_values() as u32,
+            additional,
             size,
             values,
             validity,
         ),
-        (Encoding::Plain, None, false) => {
-            read_required(page.buffer(), page.num_values() as u32, size, values)
-        }
+        // it can happen that there is a dictionary but the encoding is plain because
+        // it falled back.
+        (Encoding::Plain, _, false) => read_required(page.buffer(), additional, size, values),
         _ => {
             return Err(utils::not_implemented(
                 &page.encoding(),
